package com.example.survey.service.impl;

import com.example.survey.dao.*;
import com.example.survey.dto.record.*;
import com.example.survey.entity.*;
import com.example.survey.entity.inner.Operation;
import com.example.survey.enumeration.RecordStateEnum;

import com.example.survey.enumeration.ResultEnum;
import com.example.survey.exception.*;
import com.example.survey.service.RecordService;
import com.example.survey.util.WordUtil;
import com.example.survey.vo.RecordVO;
import com.example.survey.vo.inner.OperationInfo;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.stream.Collectors;

/**
 * @author Pope
 */
@Log4j2
@Service
public class RecordServiceImpl implements RecordService {

    @Autowired
    private RecordDao recordDao;

    @Autowired
    private UserDao userDao;

    @Autowired
    private ProjectDao projectDao;

    @Autowired
    private MetaDataDao metaDataDao;

    @Value("${file.path}")
    private String path;

    @Value("${file.url}")
    private String url;


    @Override
    public void reviewRecord(ReviewRecordDTO reviewRecordDTO) {
        if (!userDao.existUser(reviewRecordDTO.getReviewerPhone())) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        User user = userDao.selectUser(reviewRecordDTO.getReviewerPhone());
        
        if (!recordDao.existRecord(reviewRecordDTO.getUuid())) {
            throw new RecordException(ResultEnum.NOT_EXIST_RECORD);
        }
        Record record = recordDao.getRecord( reviewRecordDTO.getUuid());

        record.setState(reviewRecordDTO.getPass() ? RecordStateEnum.REVIEWED.getValue() : RecordStateEnum.NOT_PASS.getValue());
        // record.setVersion(UUID.randomUUID().toString());
        Operation reviewOp = Operation.reviewOp(user, reviewRecordDTO.getMsg(), reviewRecordDTO.getPass());
        List<Operation> opList = record.getOperationList();
        opList.add(reviewOp);
        record.setOperationList(opList);

        recordDao.saveRecord(record);
    }

    @Override
    public void modifyRecord(ModifyRecordDTO modifyRecordDTO) {
        if (!userDao.existUser(modifyRecordDTO.getUserPhone())) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        User user = userDao.selectUser(modifyRecordDTO.getUserPhone());
      
        Record record = recordDao.getRecord(modifyRecordDTO.getUuid());
        if (record == null) {
            throw new RecordException(ResultEnum.NOT_EXIST_RECORD);
        }
        List<Operation> operationList = record.getOperationList();
        Operation modifyOp = Operation.modifyOp(user, modifyRecordDTO.getMsg());
        operationList.add(modifyOp);

        record.setOperationList(operationList);
        record.setState(RecordStateEnum.UNDER_REVIEW.getValue());
        recordDao.saveRecord(record);
    }

    @Override
    public void createRecord(SubmitRecordDTO submitRecordDTO) {
        if (!userDao.existUser(submitRecordDTO.getUserPhone())) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        User user = userDao.selectUser(submitRecordDTO.getUserPhone());

        if (!metaDataDao.existMetaData(submitRecordDTO.getMetaDataName())) {
            throw new MetaDataException(ResultEnum.NOT_EXIST_METADATA);
        }
        MetaData metaData = metaDataDao.selectMetaData(submitRecordDTO.getMetaDataName());
        Record record = new Record();
        
        record.setUser(user);
        record.setValues(submitRecordDTO.getValues());
        Operation submitOp = Operation.submitOp(user, submitRecordDTO.getMsg());
        List<Operation> operations = new ArrayList<>();
        operations.add(submitOp);
        record.setOperationList(operations);
        record.setUuid(submitRecordDTO.getUuid());
        record.setIdNumber(submitRecordDTO.getIdNumber());
        record.setState(RecordStateEnum.UNDER_REVIEW.getValue());
        record.setMetaData(metaData);
        record.setMetaDataType(metaData.getType());
        record.setSubmitTime(System.currentTimeMillis());
        recordDao.saveRecord(record);
    }

    @Override
    public long countRecord(String idNumber, String state, String uuid, 
                String templateName, String templateType, 
                long submitTimeGt, long submitTimeLt, String projectName) {
       
        return recordDao.countRecord(idNumber, state, uuid, templateName, templateType, submitTimeGt, submitTimeLt, projectName);
    }

    @Override
    public List<RecordVO> listRecordLimit(String idNumber, String state, String uuid, 
            String templateName, String templateType, 
            long submitTimeGt, long submitTimeLt, String projectName, int currentPage, int pageSize) {
      
        List<Record> recordList = recordDao.listRecordLimit(idNumber, state, uuid, templateName, templateType, submitTimeGt, submitTimeLt, projectName, currentPage * pageSize, pageSize);
        return recordList.stream().map(record -> {
            RecordVO recordVO = new RecordVO();
            recordVO.setIdNumber(record.getIdNumber());
            recordVO.setUuid(record.getUuid());
            recordVO.setTemplate(record.getMetaData().getName());
            recordVO.setOperationInfoList(record.getOperationList().stream().map(op -> {
                OperationInfo operationInfo = new OperationInfo();
                operationInfo.setType(op.getType());
                operationInfo.setTime(op.getTime());
                operationInfo.setPersonId(op.getUser().getPhone());
                operationInfo.setPersonName(op.getUser().getUsername());
                operationInfo.setMsg(op.getMsg());
                operationInfo.setResult(op.getResult());
                return operationInfo;
            }).collect(Collectors.toList()));
            recordVO.setState(record.getState());
            recordVO.setSubmitTime(record.getSubmitTime());
            return recordVO;
        }).collect(Collectors.toList());
    }

    @Override
    public Map<String, Object> getRecordValues(String uuid) {
        
        Record record = recordDao.getRecord(uuid);
        if (record == null) {
            throw new RecordException(ResultEnum.NOT_EXIST_RECORD);
        }
        return record.getValues();
    }

    @Override
    public String uploadFile(MultipartFile file) {
        String name = file.getOriginalFilename();
        assert name != null;
        String suffix = name.substring(name.lastIndexOf('.'));
        String newName = UUID.randomUUID().toString() + suffix;
        File newFile = new File(path + newName);

        OutputStream os = null;
        InputStream is = null;

        try {
            os = new FileOutputStream(newFile);
            is = file.getInputStream();

            byte[] buffer = new byte[1024];
            int len;
            while ((len = is.read(buffer)) != -1) {
                os.write(buffer, 0, len);
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (os != null) {
                    os.close();
                }
                if (is != null) {
                    is.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return url + newName;
    }

    @Override
    public void deleteRecord(DeleteRecordDTO deleteRecordDTO) {
        if (!userDao.existUser(deleteRecordDTO.getPhone())) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        User user = userDao.selectUser(deleteRecordDTO.getPhone());
    
        Record record = recordDao.getRecord(deleteRecordDTO.getUuid());
        if (record == null) {
            throw new RecordException(ResultEnum.NOT_EXIST_RECORD);
        }
        List<Operation> operationList = record.getOperationList();
        Operation deleteOp = Operation.deleteOp(user, deleteRecordDTO.getMsg());
        operationList.add(deleteOp);
        record.setOperationList(operationList);
        record.setState(RecordStateEnum.FILED.getValue());
        recordDao.saveRecord(record);
    }

    @Override
    public void record2word(String uuid, HttpServletResponse response) {
        Record record = recordDao.getRecord(uuid);
        
    }

    @Override
    public void modifyMetaData(ModifyMetaDataDTO modifyMetaDataDTO) {
       
        if (!metaDataDao.existMetaData(modifyMetaDataDTO.getMetaDataName())) {
            throw new MetaDataException(ResultEnum.NOT_EXIST_METADATA);
        }
        MetaData metaData = metaDataDao.selectMetaData(modifyMetaDataDTO.getMetaDataName());
       
        Record record = recordDao.getRecord(modifyMetaDataDTO.getUuid());
        if(record == null) {
            throw new RecordException(ResultEnum.NOT_EXIST_RECORD);
        }

        if (!userDao.existUser(modifyMetaDataDTO.getPhone())) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        User user = userDao.selectUser(modifyMetaDataDTO.getPhone());

        List<Operation> operationList = record.getOperationList();
        String msg = record.getMetaData().getName() +"->" + modifyMetaDataDTO.getMetaDataName();
        Operation modifyMetaOp = Operation.modifyMetaOp(user, msg);
        // modifyMetaOp.setMsg();
        operationList.add(modifyMetaOp);
        record.setOperationList(operationList);
        record.setMetaData(metaData);
        recordDao.saveRecord(record);
    }

}
