package com.example.survey.service.impl;

import com.example.survey.dao.RoleDao;
import com.example.survey.dao.UserDao;
import com.example.survey.dto.role.CreateRoleDTO;
import com.example.survey.dto.role.DeleteRoleDTO;
import com.example.survey.dto.role.ModifyRoleDTO;
import com.example.survey.entity.Role;
import com.example.survey.enumeration.AuthEnum;
import com.example.survey.enumeration.ResultEnum;
import com.example.survey.exception.AuthException;
import com.example.survey.exception.RoleException;
import com.example.survey.service.RoleService;
import com.example.survey.util.TokenUtil;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

/**
 * @author Pope
 */
@Log4j2
@Service
public class RoleServiceImpl implements RoleService {

    @Autowired
    private RoleDao roleDao;

    @Autowired
    private UserDao userDao;

    @Override
    public void addRole(CreateRoleDTO createRoleDTO) {
        Role role = new Role();
        role.setName(createRoleDTO.getName());
        Set<AuthEnum> authoritySet = new HashSet<>();
        for (String name : createRoleDTO.getAuthoritySet()) {
            AuthEnum authEnum = AuthEnum.getRoleEnum(name);
            if (authEnum == null) {
                throw new AuthException(ResultEnum.NOT_EXIST_AUTHORITY);
            }
            authoritySet.add(authEnum);
        }
        role.setAuthoritySet(authoritySet);
        roleDao.saveRole(role);
    }

    @Override
    public List<CreateRoleDTO> listRoleLimit(String name, int currentPage, int pageSize) {
        List<CreateRoleDTO> createRoleDTOList = new LinkedList<>();
        List<Role> roleList = roleDao.listRoleLimit(name, currentPage * pageSize, pageSize);
        if (roleList == null) {
            return createRoleDTOList;
        }
        for (Role role : roleList) {
            createRoleDTOList.add(new CreateRoleDTO(role));
        }

        return createRoleDTOList;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteRole(DeleteRoleDTO deleteRoleDTO) {
        if (!roleDao.existRole(deleteRoleDTO.getName())) {
            throw new RoleException(ResultEnum.NOT_EXIST_ROLE);
        }
        //将用户数据库中含有该角色的用户更新
        userDao.clearRole(roleDao.selectRole(deleteRoleDTO.getName()));
        //从角色数据库删除角色
        roleDao.deleteRole(deleteRoleDTO.getName());
        //删除redis中的角色
        TokenUtil.expireKey(deleteRoleDTO.getName());
    }

    @Override
    public void modifyRole(ModifyRoleDTO modifyRoleDTO) {
        if (!roleDao.existRole(modifyRoleDTO.getName())) {
            throw new RoleException(ResultEnum.NOT_EXIST_ROLE);
        }
        Role role = roleDao.selectRole(modifyRoleDTO.getName());
        Set<AuthEnum> authoritySet = new HashSet<>();
        Set<String> routeSet = new HashSet<>();
        for (String authName : modifyRoleDTO.getAuthoritySet()) {
            AuthEnum authEnum = AuthEnum.getRoleEnum(authName);
            if (authEnum == null) {
                throw new AuthException(ResultEnum.NOT_EXIST_AUTHORITY);
            }
            authoritySet.add(authEnum);
            routeSet.addAll(authEnum.getRoutes());
        }
        role.setAuthoritySet(authoritySet);
        roleDao.saveRole(role);
        TokenUtil.setWithoutExpireTime(modifyRoleDTO.getName(), routeSet);
    }

    @Override
    public long countRole(String name) {
        return roleDao.countRole(name);
    }
}
