package com.example.survey.service.impl;

import com.example.survey.dao.RoleDao;
import com.example.survey.dao.UserDao;
import com.example.survey.dto.user.*;
import com.example.survey.entity.Role;
import com.example.survey.entity.User;
import com.example.survey.enumeration.AuthEnum;
import com.example.survey.enumeration.ResultEnum;
import com.example.survey.exception.RoleException;
import com.example.survey.exception.UserException;
import com.example.survey.service.UserService;
import com.example.survey.util.TokenUtil;
import com.example.survey.vo.LoginVO;
import com.example.survey.vo.RoleVO;
import com.example.survey.vo.UserVO;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

/**
 * @author Pope
 */
@Log4j2
@Service
public class UserServiceImpl implements UserService {

    @Autowired
    private UserDao userDao;
    @Autowired
    private RoleDao roleDao;


    @Override
    public LoginVO matchAuth(LoginDTO loginDTO) {
        if (!userDao.existUser(loginDTO.getPhone())) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        User user = userDao.selectUser(loginDTO.getPhone());
        if (!user.getPassword().equals(loginDTO.getPassword())) {
            throw new UserException(ResultEnum.WRONG_PASSWORD);
        }

        LoginVO loginVO = new LoginVO();
        //判断是否已经登录过
        if (TokenUtil.existKey(user.getPhone())) {
            String oldToken = (String) TokenUtil.get(user.getPhone());
            if (TokenUtil.existKey(oldToken)) {
                //已经登录，将旧token过期，
                TokenUtil.expireKey(oldToken);
                TokenUtil.expireKey(oldToken + " : USER_PHONE");
            }
        }

        //生成新的token并存入redis
        String newToken = UUID.randomUUID().toString();
        TokenUtil.set(user.getPhone(), newToken);
        TokenUtil.set(newToken + " : USER_PHONE", user.getPhone());

        //生成角色列表
        Set<String> roleNameSet = new HashSet<>();
        for (Role role : user.getRoleSet()) {
            Set<String> routeSet = new HashSet<>();
            for (AuthEnum authEnum : role.getAuthoritySet()) {
                routeSet.addAll(authEnum.getRoutes());
            }
            TokenUtil.setWithoutExpireTime(role.getName(), routeSet);
            roleNameSet.add(role.getName());
        }
        TokenUtil.set(newToken, roleNameSet);

        loginVO.setToken(newToken);
        loginVO.setUsername(user.getUsername());
        loginVO.setIdNumber(user.getIdNumber());
        loginVO.setPhone(user.getPhone());
        if (user.getRoleSet() != null) {
            loginVO.setRoleSet(user.getRoleSet().stream()
                    .map(role -> {
                        RoleVO roleVO = new RoleVO();
                        roleVO.setName(role.getName());
                        roleVO.setAuthoritySet(role.getAuthoritySet().stream()
                                .map(AuthEnum::getName)
                                .collect(Collectors.toSet())
                        );
                        return roleVO;
                    })
                    .collect(Collectors.toSet()));
        }
        loginVO.setDepartmentList(user.getDepartmentList());
        loginVO.setAdministrativeArea(user.getAdministrativeArea());

        return loginVO;
    }

    @Override
    public void addUser(CreateUserDTO createUserDTO) {
        User user = new User();
        user.setUsername(createUserDTO.getUsername());
        user.setPhone(createUserDTO.getPhone());
        user.setPassword(createUserDTO.getPassword());
        user.setIdNumber(createUserDTO.getIdNumber());
        user.setAdministrativeArea(createUserDTO.getAdministrativeArea());
        userDao.saveUser(user);
    }

    @Override
    public void modifyRole(ModifyUserRoleDTO modifyUserRoleDTO) {
        if (!userDao.existUser(modifyUserRoleDTO.getPhone())) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        User user = userDao.selectUser(modifyUserRoleDTO.getPhone());
        Set<Role> roleSet = new HashSet<>();
        for (String name : modifyUserRoleDTO.getRoleSet()) {
            if (!roleDao.existRole(name)) {
                throw new RoleException(ResultEnum.NOT_EXIST_ROLE);
            }
            roleSet.add(roleDao.selectRole(name));
        }
        user.setRoleSet(roleSet);
        userDao.saveUser(user);
    }

    @Override
    public List<UserVO> listUserLimit(String username, String phoneNumber, int currentPage, int pageSize) {
        List<User> userList = userDao.listUserLimit(username, phoneNumber, currentPage * pageSize, pageSize);
        if (userList == null) {
            return new ArrayList<>();
        }
        return userList.stream().map(user -> {
            UserVO userVO = new UserVO();
            userVO.setPhone(user.getPhone());
            userVO.setUsername(user.getUsername());
            userVO.setIdNumber(user.getIdNumber());
            userVO.setAdministrativeArea(user.getAdministrativeArea());
            if (user.getRoleSet() != null) {
                userVO.setRoleSet(user.getRoleSet().stream()
                        .map(role -> {
                            RoleVO roleVO = new RoleVO();
                            roleVO.setName(role.getName());
                            roleVO.setAuthoritySet(role.getAuthoritySet().stream()
                                    .map(AuthEnum::getName)
                                    .collect(Collectors.toSet())
                            );
                            return roleVO;
                        })
                        .collect(Collectors.toSet())
                );
            }
            return userVO;
        }).collect(Collectors.toList());
    }

    @Override
    public void deleteUser(DeleteUserDTO deleteUserDTO) {
        if (!userDao.existUser(deleteUserDTO.getPhone())) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        userDao.deleteUser(deleteUserDTO.getPhone());
        if (TokenUtil.existKey(deleteUserDTO.getPhone())) {
            String token = (String) TokenUtil.get(deleteUserDTO.getPhone());
            TokenUtil.expireKey(deleteUserDTO.getPhone());
            TokenUtil.expireKey(token);
        }
    }

    @Override
    public void modifyUserInfo(ModifyUserInfoDTO modifyUserInfoDTO) {
        if (!userDao.existUser(modifyUserInfoDTO.getPhone())) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        User user = userDao.selectUser(modifyUserInfoDTO.getPhone());
        user.setUsername(modifyUserInfoDTO.getUsername());
        user.setIdNumber(modifyUserInfoDTO.getIdNumber());
        user.setAdministrativeArea(modifyUserInfoDTO.getAdministrativeArea());
        userDao.saveUser(user);
    }

    @Override
    public void resetPwd(ResetPwdDTO resetPwdDTO) {
        if (!userDao.existUser(resetPwdDTO.getPhone())) {
            throw new UserException(ResultEnum.NOT_EXIST_USER);
        }
        User user = userDao.selectUser(resetPwdDTO.getPhone());
        user.setPassword("123456");
        userDao.saveUser(user);
    }

    @Override
    public long countUser(String username, String phone) {
        return userDao.countUser(username, phone);

    }


}
