package com.example.survey.util;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * @author Pope
 */
@Component
public class TokenUtil {

    /**
     * 过期时间6小时
     */
    private static final long EXPIRATION_TIME = 6L;

    private static TokenUtil tokenUtil;

    @Autowired
    RedisTemplate<String, Object> redisTemplate;

    @PostConstruct
    public void init() {
        tokenUtil = this;
        tokenUtil.redisTemplate = this.redisTemplate;
    }

    /**
     * 判断key是否存在
     *
     * @param key key
     * @return 是否存在
     */
    public static boolean existKey(String key) {
        Boolean result = tokenUtil.redisTemplate.hasKey(key);
        return result != null && result;
    }

    /**
     * 存储key-value键值对，如果key已存在则更新value
     * 过期时间为{@link TokenUtil#EXPIRATION_TIME}小时
     *
     * @param key   key
     * @param value value
     */
    public static void set(String key, String value) {
        tokenUtil.redisTemplate.opsForValue().set(key, value, EXPIRATION_TIME, TimeUnit.HOURS);
    }

    /**
     * 刷新key的过期时间
     *
     * @param key key
     */
    public static void refreshExpireTime(String key) {
        tokenUtil.redisTemplate.expire(key, EXPIRATION_TIME, TimeUnit.HOURS);
    }

    /**
     * 存储key-set键值对，如果key已存在则更新set
     * 过期时间为{@link TokenUtil#EXPIRATION_TIME}小时
     *
     * @param key key
     * @param set 集合
     */
    public static void set(String key, Set<String> set) {
        tokenUtil.redisTemplate.opsForValue().set(key, set, 6L, TimeUnit.HOURS);
    }

    /**
     * 存储key-set键值对，如果key已存在则更新set
     * <b>无过期时间</b>
     *
     * @param key key
     * @param set set
     */
    public static void setWithoutExpireTime(String key, Set<String> set) {
        tokenUtil.redisTemplate.opsForValue().set(key, set);
    }

    /**
     * 根据token与请求路由和请求方法判断是否允许访问
     *
     * @param token  token
     * @param route  请求的路由
     * @param method 请求方法
     * @return 是否允许访问
     */
    public static boolean isPass(String token, String route, String method) {
        if (!existKey(token)) {
            return false;
        }
        String request = route + " : " + method;
        Set<String> authSet = (Set<String>) tokenUtil.redisTemplate.opsForValue().get(token);
        if (authSet == null) {
            return false;
        }
        for (String auth : authSet) {
            Set<String> routeSet = (Set<String>) tokenUtil.redisTemplate.opsForValue().get(auth);
            if (routeSet != null && routeSet.contains(request)) {
                return true;
            }
        }
        return false;
    }

    /**
     * 根据key获取value
     *
     * @param key key
     * @return value
     */
    public static Object get(String key) {
        return tokenUtil.redisTemplate.opsForValue().get(key);
    }

    /**
     * 使某个key过期
     *
     * @param key 要过期的key
     */
    public static void expireKey(String key) {
        tokenUtil.redisTemplate.expire(key, 0, TimeUnit.MICROSECONDS);
    }
}
